package sun.java2d.windows;

import sun.awt.windows.WToolkit;
import java.security.AccessController;
import sun.java2d.opengl.WGLGraphicsConfig;
import java.security.PrivilegedAction;

public class WindowsFlags
{
    private static boolean gdiBlitEnabled;
    private static boolean d3dEnabled;
    private static boolean d3dVerbose;
    private static boolean d3dSet;
    private static boolean d3dOnScreenEnabled;
    private static boolean oglEnabled;
    private static boolean oglVerbose;
    private static boolean offscreenSharingEnabled;
    private static boolean accelReset;
    private static boolean checkRegistry;
    private static boolean disableRegistry;
    private static boolean magPresent;
    private static boolean setHighDPIAware;
    private static float autoScalingThreshold;
    private static boolean autoScaling;
    private static String javaVersion;
    private static final float DEFAULT_AUTO_SCALING_THRESHOLD = 1.5f;
    
    public WindowsFlags() {
    }
    
    private static native boolean initNativeFlags();
    
    private static native float getScaleY();
    
    public static void initFlags() {
    }
    
    private static boolean getBooleanProp(String key, boolean n) {
        String property = System.getProperty(key);
        if (property != null) {
            if (property.equals("true") || property.equals("t") || property.equals("True") || property.equals("T") || property.equals("")) {
                n = 1;
            }
            else if (property.equals("false") || property.equals("f") || property.equals("False") || property.equals("F")) {
                n = 0;
            }
        }
        return n;
    }
    
    private static boolean isBooleanPropTrueVerbose(String key) {
        String property = System.getProperty(key);
        return property != null && (property.equals("True") || property.equals("T"));
    }
    
    private static int getIntProp(String key, int int1) {
        String property = System.getProperty(key);
        if (property != null) {
            try {
                int1 = Integer.parseInt(property);
            }
            catch (NumberFormatException ex) {}
        }
        return int1;
    }
    
    private static boolean getPropertySet(String key) {
        return System.getProperty(key) != null;
    }
    
    private static void initJavaFlags() {
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                WindowsFlags.magPresent = WindowsFlags.access$100("javax.accessibility.screen_magnifier_present", false);
                int n = !WindowsFlags.access$100("sun.java2d.noddraw", WindowsFlags.magPresent);
                int access$100 = WindowsFlags.access$100("sun.java2d.ddoffscreen", (boolean)n);
                WindowsFlags.d3dEnabled = WindowsFlags.access$100("sun.java2d.d3d", n && access$100);
                WindowsFlags.d3dOnScreenEnabled = WindowsFlags.access$100("sun.java2d.d3d.onscreen", WindowsFlags.d3dEnabled);
                WindowsFlags.oglEnabled = WindowsFlags.access$100("sun.java2d.opengl", false);
                if (WindowsFlags.oglEnabled) {
                    WindowsFlags.oglVerbose = WindowsFlags.access$600("sun.java2d.opengl");
                    if (WGLGraphicsConfig.isWGLAvailable()) {
                        WindowsFlags.d3dEnabled = false;
                    }
                    else {
                        if (WindowsFlags.oglVerbose) {
                            System.out.println("Could not enable OpenGL pipeline (WGL not available)");
                        }
                        WindowsFlags.oglEnabled = false;
                    }
                }
                WindowsFlags.gdiBlitEnabled = WindowsFlags.access$100("sun.java2d.gdiBlit", true);
                WindowsFlags.d3dSet = WindowsFlags.access$900("sun.java2d.d3d");
                if (WindowsFlags.d3dSet) {
                    WindowsFlags.d3dVerbose = WindowsFlags.access$600("sun.java2d.d3d");
                }
                WindowsFlags.offscreenSharingEnabled = WindowsFlags.access$100("sun.java2d.offscreenSharing", false);
                WindowsFlags.accelReset = WindowsFlags.access$100("sun.java2d.accelReset", false);
                WindowsFlags.checkRegistry = WindowsFlags.access$100("sun.java2d.checkRegistry", false);
                WindowsFlags.disableRegistry = WindowsFlags.access$100("sun.java2d.disableRegistry", false);
                WindowsFlags.javaVersion = System.getProperty("java.version");
                if (WindowsFlags.javaVersion == null) {
                    WindowsFlags.javaVersion = "default";
                }
                else {
                    int index = WindowsFlags.javaVersion.indexOf(45);
                    if (index >= 0) {
                        WindowsFlags.javaVersion = WindowsFlags.javaVersion.substring(0, index);
                    }
                }
                String property = System.getProperty("sun.java2d.autoScaleThreshold");
                if (property != null) {
                    try {
                        WindowsFlags.autoScalingThreshold = Float.parseFloat(property);
                    }
                    catch (NumberFormatException ex) {
                        WindowsFlags.autoScalingThreshold = 1.5f;
                    }
                }
                else {
                    WindowsFlags.autoScalingThreshold = 1.5f;
                }
                String s = System.getenv("JAVA2D_DPIAWARE");
                if (s == null) {
                    s = System.getProperty("sun.java2d.dpiaware");
                }
                float access$101 = WindowsFlags.access$1700();
                if (s != null) {
                    WindowsFlags.setHighDPIAware = s.equalsIgnoreCase("true");
                    WindowsFlags.autoScaling = (!WindowsFlags.setHighDPIAware && access$101 != 1.0f);
                }
                else {
                    WindowsFlags.autoScaling = (access$101 >= WindowsFlags.autoScalingThreshold);
                    WindowsFlags.setHighDPIAware = !WindowsFlags.autoScaling;
                }
                return null;
            }
        });
    }
    
    public static boolean isD3DEnabled() {
        return WindowsFlags.d3dEnabled;
    }
    
    public static boolean isD3DSet() {
        return WindowsFlags.d3dSet;
    }
    
    public static boolean isD3DOnScreenEnabled() {
        return WindowsFlags.d3dOnScreenEnabled;
    }
    
    public static boolean isD3DVerbose() {
        return WindowsFlags.d3dVerbose;
    }
    
    public static boolean isGdiBlitEnabled() {
        return WindowsFlags.gdiBlitEnabled;
    }
    
    public static boolean isTranslucentAccelerationEnabled() {
        return WindowsFlags.d3dEnabled;
    }
    
    public static boolean isOffscreenSharingEnabled() {
        return WindowsFlags.offscreenSharingEnabled;
    }
    
    public static boolean isMagPresent() {
        return WindowsFlags.magPresent;
    }
    
    public static boolean isOGLEnabled() {
        return WindowsFlags.oglEnabled;
    }
    
    public static boolean isOGLVerbose() {
        return WindowsFlags.oglVerbose;
    }
    
    public static boolean isAutoScaling() {
        return WindowsFlags.autoScaling;
    }
    
    static boolean access$100(String s, boolean b) {
        return getBooleanProp(s, b);
    }
    
    static boolean access$600(String s) {
        return isBooleanPropTrueVerbose(s);
    }
    
    static boolean access$900(String s) {
        return getPropertySet(s);
    }
    
    static float access$1700() {
        return getScaleY();
    }
    
    static {
        WToolkit.loadLibraries();
        initJavaFlags();
        initNativeFlags();
    }
}
